@@ -12,7 +12,7 @@ MainWindow::MainWindow(QWidget *parent)
     m_pSpectrum = new SpectrumAnalyzer(this,55,40);
     m_pSpectrum->setColor(QColor(255,255,255)); //set spectrum color to white
 
-    m_pSession = new Spotify::Session(this);
+    m_pSession = new Spotify::Session(this); //create new Spotify session instance
 
     m_pAppSettings = new QSettings("qspot","qspot", this);
 
@@ -21,7 +21,6 @@ MainWindow::MainWindow(QWidget *parent)
     m_pSettingsDlg = new SettingsDialog(m_pAppSettings,this); //allocate settings dialog instance
 
     //set up the the main window menu
-
     menuBar()->addAction(tr("Settings"));
     menuBar()->addAction(tr("About"));
 
@@ -32,9 +31,18 @@ MainWindow::MainWindow(QWidget *parent)
 
     m_AboutDlg.setIconPixmap(appIconMap);
     m_AboutDlg.setWindowTitle(tr("About QSpot")),
-    m_AboutDlg.setText(tr("An open-source Qt-based Spotify client for the N900/Maemo5 (based on libopenspotify).\n\n(C) 2010 Andreas Enbacka and the QSpot Team.\n\n"
+    m_AboutDlg.setText(tr("QSpot version 0.0.5-1\n\nAn open-source Qt-based Spotify client for the N900/Maemo5 (based on libopenspotify).\n\n(C) 2010 Andreas Enbacka and the QSpot Team.\n\n"
                           "Special thanks to Markus (markusm@tmo) for assistance with the graphical design and layout."));
 
+    //init the playlist listview instance
+    m_pPlaylistView = new QListView(this);
+    m_pPlaylistModel = new QStandardItemModel(this);
+    m_pPlaylistDelegate = new ListViewDelegate(this);
+    m_pPlaylistDelegate->setSubHeaderFontPointSize(14);
+
+    m_pPlaylistView->setItemDelegate(m_pPlaylistDelegate); //connect the delegate to view
+    m_pPlaylistView->setModel(m_pPlaylistModel); //connect the model to view.
+
      //setup signal / slot connections etc.
 
     connect(m_pLoginDlg,SIGNAL(rejected()),SLOT(OnLoginDialogRejected()));
@@ -46,7 +54,7 @@ MainWindow::MainWindow(QWidget *parent)
     connect(m_pSession,SIGNAL(trackPositionUpdated(qint64)),SLOT(OnTrackPosUpdated(qint64)));
     connect(m_pSession,SIGNAL(playbackPositionUpdated(qint64)),SLOT(OnPlaybackPositionUpdated(qint64)));
     connect(m_pSession,SIGNAL(playTokenLost()),SLOT(OnPlaybackTokenLost()),Qt::QueuedConnection);
-    connect(&m_PlaylistInfo,SIGNAL(itemActivated(QListWidgetItem*)),SLOT(OnPlaylistActivated(QListWidgetItem*)));
+    connect(m_pPlaylistView,SIGNAL(activated(QModelIndex)),SLOT(OnPlaylistActivated(QModelIndex)));
     connect(menuBar(),SIGNAL(triggered(QAction*)),SLOT(OnMainMenuAction(QAction*)));
     connect(this,SIGNAL(showLogin()),SLOT(OnShowLogin()),Qt::QueuedConnection);
     connect(&m_NowPlayingIcon,SIGNAL(clicked()),SLOT(OnNowPlaying()));
@@ -63,7 +71,7 @@ MainWindow::MainWindow(QWidget *parent)
                               "QPushButton:pressed {background-image: url(/usr/share/images/icon-search-on.png);}");
     m_SearchBtn.setDisabled(true);
 
-    m_PlaylistInfo.clear();
+    m_pPlaylistModel->clear();
 
     QPixmap iconMap;
 
@@ -85,7 +93,7 @@ MainWindow::MainWindow(QWidget *parent)
 
     m_GridLayout.addWidget(&m_SearchInput,0,0,1,3);
     m_GridLayout.addWidget(&m_SearchBtn,0,3);
-    m_GridLayout.addWidget(&m_PlaylistInfo,1,0,1,-1);
+    m_GridLayout.addWidget(m_pPlaylistView,1,0,1,-1);
     m_GridLayout.addWidget(&m_NowPlayingIcon,2,0);
     m_GridLayout.addWidget(&m_NowPlayingInfo,2,1,1,1,Qt::AlignLeft);
     m_GridLayout.addWidget(&m_NowPlayingPositionInfo,2,2,1,1,Qt::AlignRight);
@@ -98,11 +106,11 @@ MainWindow::MainWindow(QWidget *parent)
     m_pTrackView = new TrackView(m_pSession,this);
 
     connect(m_pTrackView,SIGNAL(playQueueSet(QList<Spotify::Track*>)),SLOT(OnSetPlayQueue(QList<Spotify::Track*>)));
-    connect(m_pTrackView,SIGNAL(playQueuePosUpdated(qint32)),SLOT(OnPlayQueuePosUpdated(qint32)));
-    connect(m_pTrackView,SIGNAL(activeQueueViewChanged(bool)),SLOT(OnActiveQueueViewChanged(bool)));
+    connect(m_pTrackView,SIGNAL(playQueuePosUpdated(qint32,qint32)),SLOT(OnPlayQueuePosUpdated(qint32,qint32)));
+    connect(m_pTrackView,SIGNAL(activeQueueViewChanged(bool,bool)),SLOT(OnActiveQueueViewChanged(bool,bool)));
 
     m_pQueueView = new QueueView(m_pSession,this);
-    connect(m_pQueueView,SIGNAL(playQueuePosUpdated(qint32)),SLOT(OnPlayQueuePosUpdated(qint32)));
+    connect(m_pQueueView,SIGNAL(playQueuePosUpdated(qint32,qint32)),SLOT(OnPlayQueuePosUpdated(qint32,qint32)));
 
     emit showLogin(); //emit signal requesting to show login dialog
 }
@@ -154,11 +162,12 @@ void MainWindow::OnTrackPosUpdated(qint64 pos)
 
 }
 
-void MainWindow::OnActiveQueueViewChanged(bool active)
+void MainWindow::OnActiveQueueViewChanged(bool active, bool shuffleActive)
 {
     //notification that active QueueView state has changed.
     //If active is false, MainWindow owned QueueView instance should be active,
     //else the TrackView owned is active.
+    //shuffleActive parameter indicates whether shuffling mode should be active
 
     if(active)
     {
@@ -168,8 +177,8 @@ void MainWindow::OnActiveQueueViewChanged(bool active)
     {
         if(m_PlayQueue.size()>0)
         {
-            m_pQueueView->setNowPlaying(m_PlayQueue,m_currQueuePos,m_playing,m_currPlaybackPos,m_currBuffPos); //set queue (now playing) instance info
-            m_pQueueView->setActive(true);
+            m_pQueueView->setNowPlaying(m_PlayQueue,m_currQueuePos,m_playing,m_currPlaybackPos,m_currBuffPos,false,-1); //set queue (now playing) instance info
+            m_pQueueView->setActive(true,shuffleActive);
         }
     }
 }
@@ -196,10 +205,7 @@ void MainWindow::OnPlaybackPositionUpdated(qint64 pos)
                 strCurrPlaybackPos = currTrackPos.toString("h:mm:ss"); //position exceeds 60 mins
             else
                 strCurrPlaybackPos = currTrackPos.toString("m:ss");
-            if(currTrack->getDuration().hour()>0) //duration longer than 60 mins
-               strCurrTrackDuration = currTrack->getDuration().toString("h:mm:ss");
-            else
-               strCurrTrackDuration = currTrack->getDuration().toString("m:ss");
+            strCurrTrackDuration = currTrack->getDurationAsString();
             outStream << "(" << strCurrPlaybackPos << "/" << strCurrTrackDuration << ")";
             m_NowPlayingPositionInfo.setText(strCurrTrackProgressInfo);
         }
@@ -213,7 +219,7 @@ void MainWindow::OnNowPlaying()
     {
         if(m_pQueueView)
         {
-            m_pQueueView->setNowPlaying(m_PlayQueue,m_currQueuePos,m_playing,m_currPlaybackPos,m_currBuffPos); //set queue (now playing) instance info
+            m_pQueueView->setNowPlaying(m_PlayQueue,m_currQueuePos,m_playing,m_currPlaybackPos,m_currBuffPos,false,m_currShufflePos); //set queue (now playing) instance info
             m_pQueueView->show();
         }
     }
@@ -225,11 +231,12 @@ void MainWindow::OnSetPlayQueue(QList<Spotify::Track*> queue)
     m_PlayQueue = queue;
 }
 
-void MainWindow::OnPlayQueuePosUpdated(qint32 pos)
+void MainWindow::OnPlayQueuePosUpdated(qint32 pos, qint32 shufflePos)
 {
      if(pos>=0)
      {
          m_currQueuePos = pos;
+         m_currShufflePos = shufflePos;
 
          Spotify::Track* currTrack = m_PlayQueue.at(m_currQueuePos);
 
@@ -247,6 +254,7 @@ void MainWindow::OnPlayQueuePosUpdated(qint32 pos)
      else //empty now playing infobar (when playqueue has reached end)
      {
         m_currQueuePos = -1;
+        m_currShufflePos = -1;
         m_NowPlayingInfo.setText("----");
         m_NowPlayingPositionInfo.setText("");
      }
@@ -304,16 +312,17 @@ void MainWindow::OnPlaylistContainersReady(QList<Spotify::Playlist*> playlists)
 
    m_Playlists = playlists; //store list of playlists
 
-   m_PlaylistInfo.clear();
+   m_pPlaylistModel->clear();
 
    for(int pos=0; pos < playlists.size(); pos++)
    {
        Spotify::Playlist* pl = playlists.at(pos);
-       QListWidgetItem *item = new QListWidgetItem();
-       item->setIcon(listTypeIcon);
-       item->setText(pl->getName());
-       item->setData(Qt::UserRole,pos); //store associated position in playlist instance list
-       m_PlaylistInfo.addItem(item);
+       QStandardItem *item = new QStandardItem();
+       item->setData(listTypeIcon,ListViewDelegate::iconRole);
+       item->setData(pl->getName(),ListViewDelegate::headerRole);
+       item->setData(QString::number(pl->getNumTracks()) + " " + tr("tracks"),ListViewDelegate::subHeaderLeftJustifyRole);
+       item->setData(pos,Qt::UserRole); //store associated position in playlist instance list
+       m_pPlaylistModel->appendRow(item);
    }
 
    m_SearchInput.setDisabled(false);
@@ -322,17 +331,18 @@ void MainWindow::OnPlaylistContainersReady(QList<Spotify::Playlist*> playlists)
    setAttribute(Qt::WA_Maemo5ShowProgressIndicator,false);
 }
 
-void MainWindow::OnPlaylistActivated(QListWidgetItem *item)
+void MainWindow::OnPlaylistActivated(QModelIndex index)
 {
 	//Display the trackview for the activated playlist
 
-	int pos = item->data(Qt::UserRole).toInt();
+        int pos = index.data(Qt::UserRole).toInt();
 
 	Spotify::Playlist* pPlaylist = m_Playlists.at(pos);
 
         if(m_pTrackView)
         {
             m_pTrackView->setPlaylist(pPlaylist); //set active playlist for track view
+            m_pTrackView->setShuffleActive(m_pQueueView->getShuffleActive());
             m_pTrackView->show();
         }
 
