@@ -8,6 +8,10 @@
 #include <QtGui/QMenuBar>
 #include <QtGui/QAction>
 #include <QtGui/QGridLayout>
+#include <QtGui/QListView>
+#include <QStandardItemModel>
+#include <QModelIndex>
+#include <QStandardItem>
 #include <QtGui/QListWidget>
 #include <QtGui/QListWidgetItem>
 #include <QtGui/QLabel>
@@ -57,6 +61,12 @@ private:
 
     QLineEdit m_SearchInput;
 
+    QStandardItemModel *m_pPlaylistModel;
+
+    ListViewDelegate *m_pPlaylistDelegate;
+
+    QListView* m_pPlaylistView;
+
     QListWidget m_PlaylistInfo;
 
     CustomLabel m_NowPlayingIcon;
@@ -77,6 +87,8 @@ private:
 
     qint32 m_currQueuePos; //current playqueue position
 
+    qint32 m_currShufflePos; //current position in shuffle queue (when shuffle mode is active)
+
     bool m_playing; //TRUE if in playing mode
 
     qint64 m_currPlaybackPos; //playback position for currently playing track
@@ -93,19 +105,19 @@ private slots:
      void OnShowLogin();
      void OnLoggedIn(Spotify::Error error);
      void OnPlaylistContainersReady(QList<Spotify::Playlist*> playlists);
-     void OnPlaylistActivated(QListWidgetItem* item);
+     void OnPlaylistActivated(QModelIndex index);
      void OnLoginDialogRejected();
      void OnMainMenuAction(QAction* action);
      void OnPlayingState(bool state);
      void OnSetPlayQueue(QList<Spotify::Track*> queue);
-     void OnPlayQueuePosUpdated(qint32 pos);
+     void OnPlayQueuePosUpdated(qint32 pos, qint32 shufflePos);
      void OnNowPlaying();
      void OnSearch();
      void OnSearchComplete(Spotify::Search* search);
      void OnTrackPosUpdated(qint64 pos);
      void OnPlaybackPositionUpdated(qint64 pos);
      void OnPlaybackTokenLost();
-     void OnActiveQueueViewChanged(bool active);
+     void OnActiveQueueViewChanged(bool active, bool shuffleActive);
      void OnSettingsSaved();
 
 signals:
