@@ -20,7 +20,10 @@ namespace Spotify
         if( m_playlist )
         {
             if( m_name.isEmpty() )
-                m_name = sp_playlist_name( m_playlist );
+            {
+                const char* charData = sp_playlist_name( m_playlist );
+                m_name = QString::fromUtf8( charData, strlen( charData ) );
+            }
             m_numTracks = sp_playlist_num_tracks( m_playlist );
 
             if( !m_owner )
