@@ -57,14 +57,9 @@ Spotify::Session::Session( QObject* parent ):
     connect(this, SIGNAL(onSearchComplete(sp_search*)),
                     this, SLOT(OnSearchComplete(sp_search*)),Qt::QueuedConnection);
 
-    m_pProcEvtTimer = new QTimer(this);
-
     m_pAudioThread = NULL;
 
     m_preBufActive = true;
-
-    connect(m_pProcEvtTimer,SIGNAL(timeout()),this,SLOT(OnProcessThreadEvents()),Qt::QueuedConnection);
-
 }
 
 void Spotify::Session::init()
@@ -197,8 +192,6 @@ void Spotify::Session::OnPlayingState(bool state)
 void Spotify::Session::OnProcessThreadEvents()
 {
     int next_timeout = -1;
-
-    //qDebug()<<"Process events in libopenspotify main thread..";
     
    do
    {
@@ -211,7 +204,7 @@ void Spotify::Session::OnProcessThreadEvents()
 
    if(m_ProcessEvents)
    {
-	   m_pProcEvtTimer->start(next_timeout);
+       QTimer::singleShot(next_timeout,this,SLOT(OnProcessThreadEvents()));
    }
 }
 
@@ -267,6 +260,24 @@ void Spotify::Session::OnPlaybackPositionUpdated(qint64 duration)
     emit playbackPositionUpdated(m_trackPos);
 }
 
+void Spotify::Session::resetPlaybackMembers()
+{
+    //public method for resetting the playback related instance members (and removes any pending events from the playback thread)
+
+    if(m_pAudioThread)
+    {
+        m_pAudioThread->getMutex()->lock();
+        m_pAudioThread->clearFIFO();
+        m_pAudioThread->setFIFOSize(0);
+        m_pAudioThread->setBufferingPaused(false);
+        m_pAudioThread->setPlaybackPaused(false);
+        m_pAudioThread->setEndOfTrack(false);
+        qApp->removePostedEvents(this); //remove any pending events for this instance (from e.g., the playback thread)
+        m_pAudioThread->getMutex()->unlock();
+    }
+}
+
+
 Spotify::Error Spotify::Session::play( Spotify::Track* track, bool preBuffer )
 {
     m_playing = false;
@@ -311,6 +322,7 @@ Spotify::Error Spotify::Session::play( Spotify::Track* track, bool preBuffer )
             m_pAudioThread->setPlaybackPaused(false);
             m_pAudioThread->setEndOfTrack(false);
             //m_pAudioThread->setNextTrackStartPos(-1);
+            //qApp->removePostedEvents(this);
             m_pAudioThread->getMutex()->unlock();
         }
     }
