@@ -53,6 +53,8 @@ namespace Spotify
 
             void setPlaying(bool playing);
 
+            void resetPlaybackMembers();  //reset the playback related instance members (and removes any pending events from the playback thread)
+
 
             /**
              * Pointer to the Session object. Used to map
@@ -70,8 +72,6 @@ namespace Spotify
 
             bool m_DisableNotifyThread;
 
-            QTimer *m_pProcEvtTimer;
-
             bool m_preBufActive; //true if track pre-buffering is enabled
 
 
